import os
import sys
import numpy
import Tigger
import tempfile
import pyfits

from astLib.astWCS import WCS
from astropy.table import Table
from Tigger.Models import SkyModel, ModelClasses

sys.path.append('/scratch/stimela')
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]

cab = utils.readJson(CONFIG)

write_catalog = ['port2tigger', 'table']

write_opts = {}
img_opts = {}

args = []

for param in cab['parameters']:
    name = param['name']
    value = param['value']
    if value is None:
        continue
    elif value is False:
        continue
    if name == 'filename':  # positional argument
        args += ['{0}'.format(value)]
    else:
        if name != "port2tigger":
            args += ['{0}{1} {2}'.format(cab['prefix'], name, value)]
    if name in write_catalog:
        write_opts[name] = value
    else:
        img_opts[name] = value

utils.xrun(cab['binary'], args)

port2tigger = write_opts.pop('port2tigger')
outfile = write_opts.pop('table')
image = img_opts.pop('filename')

if port2tigger:
    write_opts['format'] = 'fits'

if not port2tigger:
    sys.exit(0)


# convert to data file to Tigger LSM
# First make dummy tigger model
tfile = tempfile.NamedTemporaryFile(suffix='.txt')
tfile.flush()

prefix = os.path.splitext(outfile)[0]
tname_lsm = prefix + ".lsm.html"
with open(tfile.name, "w") as stdw:
    stdw.write("#format:name ra_d dec_d i emaj_s emin_s pa_d\n")

model = Tigger.load(tfile.name)
tfile.close()


def tigger_src(src, idx):

    name = "SRC%d" % idx
    flux = ModelClasses.Polarization(float(src["int_flux"]), 0, 0, 0,
                                     I_err=float(src["err_int_flux"]))
    ra, ra_err = map(numpy.deg2rad, (float(src["ra"]), float(src["err_ra"])))
    dec, dec_err = map(numpy.deg2rad, (float(src["dec"]),
                                       float(src["err_dec"])))
    pos = ModelClasses.Position(ra, dec, ra_err=ra_err, dec_err=dec_err)
    ex, ex_err = map(numpy.deg2rad, (float(src["a"]), float(src["err_a"])))
    ey, ey_err = map(numpy.deg2rad, (float(src["b"]), float(src["err_b"])))
    pa, pa_err = map(numpy.deg2rad, (float(src["pa"]), float(src["err_pa"])))

    if ex and ey:
        shape = ModelClasses.Gaussian(ex, ey, pa, ex_err=ex_err, ey_err=ey_err, pa_err=pa_err)
    else:
        shape = None
    source = SkyModel.Source(name, pos, flux, shape=shape)
    # Adding source peak flux (error) as extra flux attributes for sources,
    # and to avoid null values for point sources I_peak = src["Total_flux"]
    if shape:
        source.setAttribute("I_peak", float(src["peak_flux"]))
        source.setAttribute("I_peak_err", float(src["err_peak_flux"]))
    else:
        source.setAttribute("I_peak", float(src["int_flux"]))
        source.setAttribute("I_peak_err", float(src["err_int_flux"]))

    return source

data = Table.read('{0}_comp.{1}'.format(outfile.split('.')[0], outfile.split('.')[-1]),
                  format=outfile.split('.')[-1])

for i, src in enumerate(data):

    model.sources.append(tigger_src(src, i))

wcs = WCS(image)
centre = wcs.getCentreWCSCoords()
model.ra0, model.dec0 = map(numpy.deg2rad, centre)

model.save(tname_lsm)
# Rename using CORPAT
utils.xrun("tigger-convert", [tname_lsm, "--rename -f"])



import os
import sys
import logging
import Crasa.Crasa as crasa

sys.path.append("/scratch/stimela")
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

cab = utils.readJson(CONFIG)

args = {}
for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue

    args[name] = value


prefix = args['imagename']
port2fits = args.pop('port2fits', True)
keep_casa_images = args.pop("keep_casa_images", False)

sys.stdout.write(repr(args))
task = crasa.CasaTask(cab["binary"], **args)
task.run()

nterms = args.get("nterms", 1)
images = ["flux", "model", "residual", "psf", "image"]
STD_IMAGES = images[:4]

convert = []
if port2fits:
    for image in images:
        img ="{:s}.{:s}".format(prefix, image)
        if image == 'flux':
            _images = [img]
        elif nterms>1:
            _images = ["%s.tt%d"%(img,d) for d in range(nterms)]
            if image=="image":
                if  nterms==2:
                    alpha = img+".alpha"
                    alpha_err = img+".alpha.error"
                    _images += [alpha,alpha_err]
                if  nterms==3:
                    beta = img+".beta"
                    beta_err = img+".beta.error"
                    _images += [beta,beta_err]
        else:
            _images = [img]
        convert += _images

for _image in  convert:
    sys.stdout.write(_image)
    if _image in STD_IMAGES and (not os.path.exists(_image)):
        raise RuntimeError("Standard output from CASA clean task not found. Something went wrong durring cleaning, take a look at the logs and such")

    elif os.path.exists(_image):
        task = crasa.CasaTask("exportfits", **dict(imagename=_image, fitsimage=_image+".fits", overwrite=True))
        task.run()

if not keep_casa_images:
    for _image in convert:
        utils.xrun("rm", ["-rf", _image])



import subprocess
import os
import sys
from stimela import utils
import json
import stimela
import time
import datetime
import tempfile

class SingularityError(Exception):
    pass


def pull(image, store_path, docker=True):
    """ 
        pull an image
    """

    if docker:
        fp = "docker://{0:s}".format(image)
    else:
        fp = image
    utils.xrun("singularity", ["pull", "--force", "--name", store_path, fp])

    return 0


class Container(object):
    def __init__(self, image, name, 
                 volumes=None,
                 logger=None,
                 time_out=-1,
                 runscript=""):
        """
        Python wrapper to singularity tools for managing containers.
        """
    
        self.image = image
        self.name = name
        self.volumes = volumes or []
        self.logger = logger
        self.status = None
        self.WORKDIR = None
        self.RUNSCRIPT = runscript
        self.PID = os.getpid()
        self.uptime = "00:00:00"
        self.time_out = time_out
        #self.cont_logger = utils.logger.StimelaLogger(log_container or stimela.LOG_FILE)


    def add_volume(self, host, container, perm="rw", noverify=False):

        if os.path.exists(host) or noverify:
            if self.logger:
                self.logger.debug("Mounting volume [{0}] in container [{1}] at [{2}]".format(host, self.name, container))
            host = os.path.abspath(host)
        else:
            raise IOError("Path {0} cannot be mounted on container: File doesn't exist".format(host))
        
        self.volumes.append(":".join([host,container,perm]))

        return 0


    def start(self, *args):
        """
        Create a singularity container instance
        """

        if self.volumes: 
            volumes = " --bind " + " --bind ".join(self.volumes)
        else:
            volumes = ""
        
        self._print("Instantiating container [{0:s}]. Timeout set to {1:d}. The container ID is printed below.".format(self.name, self.time_out))
        utils.xrun("singularity instance.start", 
                        list(args) + [volumes,
                        "-c",
                        self.image, self.name])

        self.status = "created"

        return 0
        

    def run(self, *args):
        """
        Run a singularity container instance
        """

        if self.volumes: 
            volumes = " --bind " + " --bind ".join(self.volumes)
        else:
            volumes = ""
        
        self._print("Starting container [{0:s}]. Timeout set to {1:d}. The container ID is printed below.".format(self.name, self.time_out))
        utils.xrun("singularity run", ["instance://{0:s} {1:s}".format(self.name, self.RUNSCRIPT)],
                   timeout= self.time_out, kill_callback=self.stop)

        self.status = "running"

        return 0


    def stop(self, *args):
        """
        Stop a singularity container instance
        """

        if self.volumes: 
            volumes = " --bind " + " --bind ".join(self.volumes)
        else:
            volumes = ""
        
        self._print("Stopping container [{}]. The container ID is printed below.".format(self.name))
        utils.xrun("singularity", ["instance.stop {0:s}".format(self.name)])

        self.status = "exited"

        return 0


    def _print(self, message):
        if self.logger:
            self.logger.info(message)
        else:
            print(message)
            
        return 0



import os
import sys

sys.path.append('/scratch/stimela')
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]

cab = utils.readJson(CONFIG)
args = []

for param in cab['parameters']:
    name = param['name']
    value = param['value']
    if value is None:
        continue
    elif value is False:
        continue

    if value is bool:
        args += ['{0}{1}'.format(cab['prefix'], name)]
    else:
        args += ['{0}'.format(value)]

utils.xrun(cab['binary'], args)



import os
import sys

sys.path.append("/scratch/stimela")
import utils


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]
OUTDIR = os.environ["OUTPUT"]
HOME = os.environ["HOME"]

cab = utils.readJson(CONFIG)
args = []
overwrite = False

for param in cab['parameters']:
    value = param['value']
    name = param['name']

    if value in [None, False]:
        continue
    elif name == "overwrite":
        overwrite = value
        continue
    elif value is True:
        value = ""
    elif name == 'mvffiles':
        files = value
        continue
    elif name == "credentials_dir" and value:
        os.system("cp -rf {0:s} {1:s}/.aws".format(value, HOME))
        continue 
    args += ['{0}{1} {2}'.format(cab['prefix'], name, value)]


if overwrite:
    os.system("rm -fr {0:s}".format(" ".join(files)))

utils.xrun(cab["binary"], args+files )



class dismissable:
    '''
    Wrapper for optional parameters to stimela
    Initialize with val == None to force stimela to skip
    parsing parameter.
    '''
    def __init__(self, val = None):
        self.__val = val
    def __call__(self):
        return self.__val



import subprocess
import os
import sys
from io import StringIO
from stimela import utils
import json
import stimela
import time
import datetime
import tempfile

class DockerError(Exception):
    pass


def build(image, build_path, tag=None, build_args=None, fromline=None, args=[]):
    """ build a docker image"""

    if tag:
        image = ":".join([image, tag])

    bdir = tempfile.mkdtemp()
    os.system('cp -r {0:s}/* {1:s}'.format(build_path, bdir))
    if build_args:
        stdw = tempfile.NamedTemporaryFile(dir=bdir, mode='w')
        with open("{}/Dockerfile".format(bdir)) as std:
            dfile = std.readlines()

        for line in dfile:
            if fromline and line.lower().startswith('from'):
                stdw.write('FROM {:s}\n'.format(fromline))
            elif line.lower().startswith("cmd"):
                for arg in build_args:
                    stdw.write(arg+"\n")
                stdw.write(line)
            else:
                stdw.write(line)
        stdw.flush()
        utils.xrun("docker build", args+["--force-rm","-f", stdw.name, 
                   "-t", image, 
                    bdir])

        stdw.close()
    else:
        utils.xrun("docker build", args+["--force-rm", "-t", image, 
                    bdir])

    os.system('rm -rf {:s}'.format(bdir))
    
def pull(image, tag=None):
    """ pull a docker image """
    if tag:
        image = ":".join([image, tag])

    utils.xrun("docker pull", [image])


def seconds_hms(seconds):
    return str(datetime.timedelta(seconds=seconds))


class Container(object):
    def __init__(self, image, name, 
                 volumes=None, environs=None,
                 label="", logger=None, 
                 shared_memory="1gb",
                 time_out=-1,
                 log_container=None):
        """
        Python wrapper to docker engine tools for managing containers.
        """
    
        self.image = image
        self.name = name
        self.label = label
        self.volumes = volumes or []
        self.environs = environs or []
        self.logger = logger
        self.status = None
        self.WORKDIR = None
        self.COMMAND = None
        self.shared_memory = shared_memory
        self.PID = os.getpid()
        self.uptime = "00:00:00"
        self.time_out = time_out
        self.cont_logger = utils.logger.StimelaLogger(log_container or stimela.LOG_FILE)


    def  add_volume(self, host, container, perm="rw"):

        if os.path.exists(host):
            if self.logger:
                self.logger.debug("Mounting volume [{0}] in container [{1}] at [{2}]".format(host, self.name, container))
            host = os.path.abspath(host)
        else:
            raise IOError("Directory {0} cannot be mounted on container: File doesn't exist".format(host))
        
        self.volumes.append(":".join([host,container,perm]))


    def add_environ(self, key, value):
        if self.logger:
            self.logger.debug("Adding environ varaible [{0}={1}] in container {2}".format(key, value, self.name))
        self.environs.append("=".join([key, value]))


    def create(self, *args):

        if self.volumes: 
            volumes = " -v " + " -v ".join(self.volumes)
        else:
            volumes = ""
        if self.environs:
            environs = environs = " -e "+" -e ".join(self.environs)
        else:
            environs = ""
        
        self._print("Instantiating container [{}]. The container ID is printed below.".format(self.name))
        utils.xrun("docker create", list(args) + [volumes, environs,
                        "-w %s"%(self.WORKDIR) if self.WORKDIR else "",
                        "--name", self.name, "--shm-size", self.shared_memory,
                        self.image,
                        self.COMMAND or ""])

        self.status = "created"

    def info(self):

        output = subprocess.check_output("docker inspect {}".format(self.name), shell=True).decode()
        output_file = StringIO(output[3:-3])
        jdict = json.load(output_file)
        output_file.close()

        return jdict
    
    
    def get_log(self):
        stdout = open(self.logfile, 'w')
        exit_status = subprocess.call("docker logs {0}".format(self.name),
                            stdout=stdout, stderr=stdout, shell=True)
        if exit_status !=0:
            self.logger.warn('Could not log container: {}. Something went wrong durring execution'.format(self.name))
            output = 'Task was not started.'
            stdout.write(output)
        else:
            output = stdout.read()

        stdout.close()
        return output

        
    def start(self):
        running = True
        tstart = time.time()
        self.status = "running"
        
        self.cont_logger.log_container(self.name)
        self.cont_logger.write()
        try:
            self._print("Starting container [{0:s}]. Timeout set to {1:d}. The container ID is printed below.".format(self.name, self.time_out))
            utils.xrun("docker", ["start", "-a", self.name], 
                       timeout=self.time_out, 
                       kill_callback=lambda: utils.xrun("docker", ["kill", self.name]))
        except KeyboardInterrupt:
            utils.xrun("docker", ["kill", self.name])
            raise 
           
        uptime = seconds_hms(time.time() - tstart)
        self.uptime = uptime
        self._print("Container [{0}] has executed successfully".format(self.name))

        self._print("Runtime was {0}.".format(uptime))
        
        self.status = "exited"


    def stop(self):
        dinfo = self.info()
        status = dinfo["State"]["Status"]
        killed = False
        if status in ["running", "paused"]:
            try:
                utils.xrun("docker stop", [self.name])
            except KeyboardInterrupt("Received terminate signal. Will stop and remove container first"):
                killed = True
        self.status = 'exited'

        self._print("Container {} has been stopped.".format(self.name))
        if killed:
            self.remove()
            raise KeyboardInterrupt


    def remove(self):
        dinfo = self.info()
        status = dinfo["State"]["Status"]
        killed = False
        if status == "exited":
            try:
                utils.xrun("docker rm", [self.name])
            except KeyboardInterrupt:
                killed = True
            if killed:
                raise KeyboardInterrupt
           
        else:
            raise DockerError("Container [{}] has not been stopped, cannot remove".format(self.name))

        self.cont_logger.remove('containers', self.name)
        self.cont_logger.write()

    def _print(self, message):
        if self.logger:
            self.logger.info(message)
        else:
            print(message)



import os
import sys

sys.path.append("/scratch/stimela")
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]

cab = utils.readJson(CONFIG)
params = cab["parameters"]

args = []
for param in params:
    if param['value'] in [False, None]:
        continue
    elif param['value'] is True:
        arg = "{0}{1}".format(cab["prefix"], param["name"])
    else:
        arg = "{0}{1} {2}".format(cab["prefix"], param["name"], param["value"])

    args.append(arg)

utils.xrun(cab["binary"], args)



import sys
import os
import json

sys.path.append("/scratch/stimela")
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

cab = utils.readJson(CONFIG)

args = []
parset = None
for param in cab['parameters']:
    name = param['name']
    value = param['value']
    if name == 'Parset' and value is not None:
        parset = value
        continue
    if name == 'Parset' and value is None:
        continue

    if isinstance(value, list):
        arg = "{0}{1} {2}".format(cab['prefix'], name, ",".join(value))
    else:
        arg = '{0}{1} {2}'.format(cab['prefix'], name, value)

    args.append(arg)

if parset is not None:
    args.insert(0, parset)
utils.xrun(cab['binary'], args)



import matplotlib
matplotlib.use('Agg')

import numpy as np
from pyrap.tables import table
import sys,os
import pylab as plt
from optparse import OptionParser
import time
matplotlib.rcParams.update({'font.size': 22})

def info(string):
  t = "%d/%d/%d %d:%d:%d"%(time.localtime()[:6])
  print "%s ##INFO: %s"%(t,string)
def abort(string):
  t = "%d/%d/%d %d:%d:%d"%(time.localtime()[:6])
  print "%s ##ABORT: %s"%(t,string)
  sys.exit()

def taper(msname, wc=None,res=None,freq=None, savefig=None):
  tab = table(msname, readonly=False)

  bt = lambda w,wc,ws,n: np.sqrt( 1./( 1 + (wc/(w-ws))**(2.0*n) ) ) # using butterworth function to create weights
  sign = -1.0

  if len(res or [0])>1 or len(wc or [0] )>1:
    if wc is None:
      res = np.array(res)
      res = (res/3600.) * np.pi/180. # Convert resolution to radians
      wc = 1.22*((2.998e8/freq)/res)/2. # Zen master of the universe (Oleg) says divide by 2
      wc = wc[np.argsort(wc)]
    ws = (wc[1] - wc[0])/2. + wc[0]
    wc = (wc[1] - wc[0])/2.
  else:

   ws,sign = 0,1.0

   if wc is None:
     try: 
      res = res[0]
      res = (res/3600.) * np.pi/180 # Convert resolution to radians
     except TypeError: abort("Either of res or wc has to be specified")
     wc = 1.22*((2.998e8/freq)/res)/2. # Zen master of the universe (Oleg) says divide by 2
   else: 
       wc = wc[0]

  uvw = tab.getcol("UVW")
  uvdist = np.sqrt(uvw[:,0]**2 + uvw[:,1]**2)
  uvdist[np.where(uvdist==0)] = 1e-3 # avoid division by zero
  info("Calculating taper weights")
  weight_I = bt(uvdist,wc,ws,sign*20)

  if savefig:
    plt.clf()
    plt.plot(uvdist/1e3, weight_I, ".", ms=0.5)
    plt.xlabel('|uv| [km]')
    plt.ylabel('uv-weight')
    plt.ylim(0, 1.05)
    plt.grid()
    plt.xscale("log")
    plt.savefig(savefig)

  weight = np.array([weight_I,weight_I,weight_I,weight_I]).T # use same weights for all four stkes parameters [I,Q,U,V]
  nrows = len(uvw)
  rowchunk = nrows/10
  info("Inserting weights in measurement set")

  for row0 in range(0,nrows,rowchunk):
    nr = min(rowchunk,nrows-row0)
    info("Adding weights to (rows %d to %d)"%(row0,row0+nr-1))
    tab.putcol("WEIGHT",weight[row0:(row0+nr)],row0,nr) # update MS with new weights

  info("Done!")
  tab.close()

def reset_weights(msname):
  tab = table(msname, readonly=False)

  weights = tab.getcol("WEIGHT")
  weights[...] = 1
  nrows = len(weights)
  rowchunk = nrows/10
  info("Reseting weights in \'WEIGHT\' column in MS")

  for row0 in range(0,nrows,rowchunk):
    nr = min(rowchunk,nrows-row0)
    info("Reseting (rows %d to %d)"%(row0,row0+nr-1))
    tab.putcol("WEIGHT",weights[row0:(row0+nr)],row0,nr) # update MS with new weights

  tab.close()
  info("Done!")



import os
import sys
from pyrap.tables import table

sys.path.append("/scratch/stimela")
import utils
import numpy

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]
CODE = "/scratch/code/"

cab = utils.readJson(CONFIG)

_params = cab['parameters']

params = {}
options = {}
for param in _params:
    value = param['value']
    name = param['name']

    if value is None:
        continue

    params[name] = value

options["ms_sel.ddid_index"] = params.get('spw-id', 0)
options["ms_sel.field_index"] = params.get('field-id', 0)

tdlconf = params.get("tdlconf", None) or  CODE+"/tdlconf.profiles"
section = params.get("section", None) or 'sim'
mode = params.pop("mode", None) or "simulate"
threads = params.pop("threads", 4)
skymodel = params.pop("skymodel", None)
beam_files_pattern = params.pop("beam-files-pattern", False)

if isinstance(skymodel, str):
    if os.path.exists(skymodel):
        options['me.sky.tiggerskymodel'] = 1
        options["tiggerlsm.filename"] = skymodel
    else:
        raise RuntimeError("ABORT: Could not find the skymodel")

modes = {
    "simulate"  :   'sim only',
    "add"       :   "add to MS",
    "subtract"  :   'subtract from MS',
}

column = params.pop("column", "CORRECTED_DATA")
incol = params.pop("input-column", "CORRECTED_DATA")

msname = params['msname']
options["ms_sel.msname"] = msname
options["sim_mode"] = modes[mode]
options["ms_sel.input_column"] = incol
options["ms_sel.output_column"] = column
options["me.use_smearing"] = 1 if params.pop('smearing', False) else 0
saveconf = params.pop('save-config', None)

addnoise = params.pop("addnoise", False)
options["ms_sel.tile_size"] = params.pop("tile-size", 16)

if addnoise:
    noise = params.pop("noise", 0) or utils.compute_vis_noise(msname, params.pop("sefd", 551))
    options["noise_stddev"] = noise

gjones = params.pop("Gjones", False)
if gjones:
    gain_opts = {
        "me.g_enable"   :   1,
        "oms_gain_models.err-gain.error_model"  :   "SineError",
        "oms_gain_models.err-gain.max_period"   : params.pop("gain-max-period", 2),
        "oms_gain_models.err-gain.maxval"   : params.pop("gain-max-error", 1.2),
        "oms_gain_models.err-gain.min_period"   : params.pop("gain-min-period", 1),
        "oms_gain_models.err-gain.minval"   : params.pop("gain-min-error", 0.8),
    }

    options.update(gain_opts)

    phase_opts = {
        "oms_gain_models.err-phase.error_model" : "SineError",
        "oms_gain_models.err-phase.max_period"  : params.pop("phase-max-period", 2),
        "oms_gain_models.err-phase.maxerr"  : params.pop("phase-max-error", 30.0),
        "oms_gain_models.err-phase.min_period"  : params.pop("phase-min-period", 1),
        "oms_gain_models.err-phase.minval"  : params.pop("phase-min-error", 5),
    }
    
    options.update(phase_opts)

beam = params.pop("Ejones", False)
if beam and beam_files_pattern:
    beam_opts = {
        "me.e_enable"   : 1,
        "me.p_enable"   : 1,
        "me.e_module"   : "Siamese_OMS_pybeams_fits",
        "pybeams_fits.sky_rotation"    :  1 if params.pop('parallactic-angle-rotation', False) else 0,
        "me.e_all_stations" : 1,
        "pybeams_fits.l_axis"   : params.pop("beam-l-axis", "L"),
        "pybeams_fits.m_axis"   : params.pop("beam-m-axis", "M"),
        "pybeams_fits.filename_pattern" : "'{}'".format(beam_files_pattern),
    }
    options.update(beam_opts)

    rms_perr = params.get("pointing-accuracy", 0)
    # Include pointing errors if needed
    if rms_perr:
        anttab = table(msname + "/" + "ANTENNA")
        NANT = anttab.nrows()
        
        options["me.epe_enable"] = 1
        perr = numpy.random.randn(NANT)*rms_perr, numpy.random.randn(NANT)*rms_perr
        ll, mm = " ".join( map(str, perr[0]) ), " ".join( map(str, perr[-1]) )
        options['oms_pointing_errors.pe_l.values_str'] = "'%s'"%ll
        options['oms_pointing_errors.pe_l.values_str'] = "'%s'"%mm

field_center = params.pop("field-center", None)
if field_center and skymodel:
    if field_center.lower() == "ms":
        ftab = table(msname+"/FIELD")
        ra,dec = ftab.getcol("PHASE_DIR")[params.get('field-id', 0)][0]
        field_center = "J2000,%frad,%frad"%(ra, dec)
    tmp = "recentered_"+os.path.basename(skymodel)

    utils.xrun("tigger-convert", ["--recenter", field_center, skymodel, tmp, "-f"])
    options["tiggerlsm.filename"] = tmp

prefix = ['-s {}'.format(saveconf) if saveconf else ''] + ["--mt {0} -c {1} [{2}]".format(threads, tdlconf, section)]
suffix = ["%s/Siamese/turbo-sim.py =_simulate_MS"%os.environ["MEQTREES_CATTERY_PATH"]]

args = []
for key,value in options.iteritems():
    if isinstance(value, str) and value.find(' ')>0:
        value = '"{:s}"'.format(value)
    args.append('{0}={1}'.format(key,value))

utils.xrun(cab['binary'], prefix + args + suffix)



import os
import sys
import re
import pyfits
from pyrap.tables import table

sys.path.append("/scratch/stimela")
import utils

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

cab = utils.readJson(CONFIG)
params = {}
for param in cab['parameters']:
    params[param['name']] = param['value']

image = params.pop("image", None)
mask = params.pop("mask-image", None)
in_spi_image = params.pop("input-spi-image", None)
in_spi_err = params.pop("input-spi-error-image", None)
out_spi_image = params.pop("output-spi-image", None)
out_spi_err = params.pop("output-spi-error-image", None)
lsmname = params.pop("input-skymodel", None)
outlsm = params.pop("output-skymodel", None)
sigma = params.pop("sigma-level", 20) or 20
freq0 = params.pop("freq0", 0)
tol = params.pop("tolerance-range", None) or (-10,10)

if out_spi_image:
    import specfit
    specfit.spifit(image, mask=mask, sigma=sigma,
                  spi_image=out_spi_image,
                  spi_err_image=out_spi_err)
    spi_image = out_spi_image
    spi_err = out_spi_err
else:
    spi_image = in_spi_image
    spi_err = in_spi_err

if lsmname and outlsm:
    sys.stdout.write("Extracting spi from image")

    with pyfits.open(image) as hdu:
        header = hdu[0].header
        bmin = header['BMIN']
        bmaj = header['BMAJ']
        bpa = header['BPA']
    beam = (bmin, bmaj, bpa)

    import addSPI
    addSPI.addSPI(spi_image, spi_err, lsmname=lsmname, outfile=outlsm,
                  beam=beam, freq0=freq0, spitol=tol)



import os
import sys
import subprocess
sys.path.append('/scratch/stimela')
import utils
#import montage_wrapper as montage


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]
OUTPUT = os.environ["OUTPUT"]

cab = utils.readJson(CONFIG)
args = {}
for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue
    #if name == "output_dir":
    #    args["output_dir"] = os.path.join(OUTPUT, value)
    #if name == 'input_dir':
    #    args['input_dir'] = os.path.join(INPUT,value)
    args[name] = value

if os.path.exists(OUTPUT+'/mask_mosaic')==False:
    os.mkdir(OUTPUT+'/mask_mosaic')

outdir = OUTPUT+'/mask_mosaic'


make_table = ['mImgtbl',args['input_dir'],outdir+'/mosaic_table.tbl'] 
subprocess.check_output(make_table)

make_header = ['mMakeHdr',outdir+'/mosaic_table.tbl',outdir+'/mosaic_header.hdr']
subprocess.check_output(make_header)

project_mosaic = ['mProjExec', '-p',args['input_dir'],outdir+'/mosaic_table.tbl',outdir+'/mosaic_header.hdr',outdir,outdir+'/stats.tbl']
subprocess.check_output(project_mosaic)

make_mosaic_table = ['mImgtbl',outdir,outdir+'/mosaic_table2.tbl']
subprocess.check_output(make_mosaic_table)

make_mosaic=['mAdd', '-p', args['input_dir'], outdir+'/mosaic_table2.tbl',outdir+'/mosaic_header.hdr',OUTPUT+'/mosaic.fits']
subprocess.check_output(make_mosaic)

#command = 'mProjExec '+OUTPUT+'/mosaic_table.tbl '+OUTPUT+'/mosaic_header.hdr '+args['output_dir']+' '+OUTPUT+'/mosaic_stats.tbl'
#print command
#os.system(command)
